﻿using System;
using System.Windows.Forms;
using Plugininterface; // Riferimento a plugininterface.dll

namespace Plugins
{
    public class UCCNCplugin : IPluginInterface
    {
        private Plugininterface.Entry UC;

        // Variabile per la deadzone, caricata e salvata dal profilo
        private double deadzone = 2.0;
        public double Deadzone
        {
            get => deadzone;
            set => deadzone = value;
        }

        // Inizializzazione
        public void Init_event(Plugininterface.Entry UC)
        {
            this.UC = UC;

            // LED 503 acceso per indicare plugin attivo
            //UC.SetLED(true, 503);
            UC.AddStatusmessage("Virtual THC : avviato");

            // Carica la deadzone dal profilo
            LoadDeadzone();

            // Carichiamo i valori salvati nei campi 20000, 20002
            int[] fields = { 20000, 20001, 20002 };                                                // For Debug
            //int[] fields = { 20000, 20002 };                                                       // For Release
            foreach (int f in fields)
            {
                // Leggiamo il valore salvato nel profilo
                string storedVal = UC.Readkey("UserTextfields", f.ToString(), "0");
                if (!double.TryParse(storedVal, out double val))
                    val = 0;

                // Impostiamo il campo in UCCNC
                UC.Setfield(true, val, f);
                UC.Validatefield(true, f);
            }
        }

        // Proprietà del plugin
        public Entry.Pluginproperties Getproperties_event(Entry.Pluginproperties Properties)
        {
            Properties.author = "CS-Tec";
            Properties.pluginname = "VirtualTHC Debug";                                            // For Debug
            //Properties.pluginname = "VirtualTHC";                                                  // For Release
            Properties.pluginversion = "1.0.0";
            return Properties;
        }

        // Avvio del plugin
        public void Startup_event()
        {
            // Se vuoi, potresti mostrare un messaggio o altro
            // Esempio:
            //UC.AddStatusmessage("VirtualTHC avviato");

        }

        // Chiusura del plugin
        public void Shutdown_event()
        {
            // LED 503 spento per indicare plugin fermo
            //UC.SetLED(false, 503);
            UC.AddStatusmessage("Virtual THC : fermato");
        }

        // Metodo chiamato se l'utente preme "Configure" nei plugin
        public void Configure_event()
        {
            using (var cfg = new ConfigForm(this))
            {
                cfg.ShowDialog();
            }
        }

        // Metodo chiamato se l'utente preme "Show" (Pluginshowup)
        public void Showup_event()
        {
            // Se non hai un form principale, lascia pure vuoto.
            // Oppure potresti mostrare un form di debug.
            MessageBox.Show("Non ho un form principale, uso solo 'Configure'.");
        }

        // Ciclo principale (25 Hz)
        public void Loop_event()
        {
            try
            {
                // Legge il setpoint dal campo 20000
                double setpoint = Convert.ToDouble(UC.Getfield(true, 20000));

                // Legge il valore (in BIT)
                double analogBits = Convert.ToDouble(UC.Getfield(true, 20001));                    // For Debug
                //double analogBits = Convert.ToDouble(UC.Getanaloginput(1));                        // For Release

                // Converte i bit in tensione
                double voltage = Math.Round(analogBits * ((50.0 * 5.0) / 65535.0), 0);

                // Scriviamo il valore convertito in 20002 (debug)
                UC.Setfield(true, voltage, 20002);
                UC.Validatefield(true, 20002);

                // Lettura LED
                bool ledTorch = UC.GetLED(50);   // Torcia
                bool ledArc = UC.GetLED(237); // ARC
                bool ledUp = UC.GetLED(238); // THC Up
                bool ledDown = UC.GetLED(239); // THC Down

                // Logica Virtual THC
                if (voltage > 50 && ledTorch)
                {
                    // Se ARC non è attivo, attivalo
                    if (!ledArc)
                    {
                        UC.Callbutton(536); // ARC ON
                    }

                    // Se tensione > setpoint + deadzone/2
                    if (voltage > (setpoint + (deadzone / 2)) && setpoint > 0)
                    {
                        if (!ledDown)
                        {
                            UC.Callbutton(539); // Disattiva THC Up
                            UC.Callbutton(540); // Attiva THC Down
                        }
                    }
                    // Se tensione < setpoint - deadzone/2
                    else if (voltage < (setpoint - (deadzone / 2)) && setpoint > 0)
                    {
                        if (!ledUp)
                        {
                            UC.Callbutton(541); // Disattiva THC Down
                            UC.Callbutton(538); // Attiva THC Up
                        }
                    }
                    else
                    {
                        // Se entro la zona morta, disattiva Up/Down
                        if (ledUp || ledDown)
                        {
                            UC.Callbutton(539); // Disattiva THC Up
                            UC.Callbutton(541); // Disattiva THC Down
                        }
                    }
                }
                else
                {
                    // Se tensione <= 50 o LED Torcia spento, spegni ARC e disattiva THC
                    if (ledArc || ledUp || ledDown)
                    {
                        UC.Callbutton(537); // ARC OFF
                        UC.Callbutton(539); // Disattiva THC Up
                        UC.Callbutton(541); // Disattiva THC Down
                    }
                }
            }
            catch
            {
                // Se c'è un errore, ignoriamo o potresti spegnere LED 503, ma senza popup
                //UC.SetLED(false, 503);
                UC.AddStatusmessage("Virtual THC : errore");
            }
        }

        // Funzione di utilità: Carica la deadzone dal profilo
        private void LoadDeadzone()
        {
            string saved = UC.Readkey("VirtualTHC", "Deadzone", "2.0");
            if (!double.TryParse(saved, out deadzone))
                deadzone = 2.0;
        }

        // Funzione di utilità: Salva la deadzone nel profilo
        public void SaveDeadzone()
        {
            UC.Writekey("VirtualTHC", "Deadzone", deadzone.ToString());
        }

        // Metodo per comunicare con altri plugin (opzionale)
        public object Informplugin_event(object Message)
        {
            return "Virtual THC: messaggio ricevuto, ciao!";
        }
    }
}
